# Executed when running
# > python sclblonnx
# from terminal.
if __name__ == '__main__':
    print("The sclblonnx package currently does not have any command line options.")
    print("Please import the package into your python project and work from there.")
